"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const template_1 = __importDefault(require("./template"));
const slide_1 = __importDefault(require("./slide"));
const file_1 = __importDefault(require("./helper/file"));
class Automizer {
    /**
     * Parameters for Automizer constructor.
     * @param {string} templateDir - optional: prefix for all templates to load.
     * @param {string} outputDir - optional: prefix for all output files.
     */
    constructor(params) {
        this.templates = [];
        this.params = params;
        this.templateDir = (params === null || params === void 0 ? void 0 : params.templateDir) ? params.templateDir + '/' : '';
        this.outputDir = (params === null || params === void 0 ? void 0 : params.outputDir) ? params.outputDir + '/' : '';
        this.timer = Date.now();
    }
    /**
     * Load a pptx file and set it as root template.
     * @param {string} location - Filename or path to the template. Will be prefixed with 'templateDir'
     * @return {Automizer} Instance of Automizer
     */
    loadRoot(location) {
        return this.loadTemplate(location);
    }
    /**
     * Load a template pptx file.
     * @param {string} location - Filename or path to the template. Will be prefixed with 'templateDir'
     * @param {string} name - Optional: A short name for the template. If skipped, the template will be named by its location.
     * @return {Automizer} Instance of Automizer
     */
    load(location, name) {
        name = (name === undefined) ? location : name;
        return this.loadTemplate(location, name);
    }
    loadTemplate(location, name) {
        location = this.getLocation(location, 'template');
        let newTemplate = template_1.default.import(location, name);
        if (!this.isPresTemplate(newTemplate)) {
            this.rootTemplate = newTemplate;
        }
        else {
            this.templates.push(newTemplate);
        }
        return this;
    }
    isPresTemplate(template) {
        return 'name' in template;
    }
    /**
     * Find imported template by given name and return a certain slide by number.
     * @param {string} name - Name of template; must be imported by Automizer.importTemplate()
     * @param {number} slideNumber - Number of slide in template presentation
     * @return {Automizer} Instance of Automizer
     */
    addSlide(name, slideNumber, callback) {
        if (this.rootTemplate === undefined) {
            throw new Error('You have to set a root template first.');
        }
        let template = this.template(name);
        let newSlide = new slide_1.default({
            presentation: this,
            template: template,
            number: slideNumber
        });
        if (callback !== undefined) {
            newSlide.root = this,
                callback(newSlide);
        }
        this.rootTemplate.slides.push(newSlide);
        return this;
    }
    template(name) {
        let template = this.templates.find(template => template.name === name);
        if (template === undefined) {
            throw new Error(`Template not found: ${name}`);
        }
        return template;
    }
    getLocation(location, type) {
        switch (type) {
            case 'template':
                return this.templateDir + location;
            case 'output':
                return this.outputDir + location;
            default:
                return location;
        }
    }
    write(location) {
        return __awaiter(this, void 0, void 0, function* () {
            let rootArchive = yield this.rootTemplate.archive;
            for (let i in this.rootTemplate.slides) {
                let slide = this.rootTemplate.slides[i];
                yield this.rootTemplate.appendSlide(slide);
            }
            let content = yield rootArchive.generateAsync({ type: "nodebuffer" });
            return file_1.default.writeOutputFile(this.getLocation(location, 'output'), content, this);
        });
    }
}
exports.default = Automizer;
//# sourceMappingURL=automizer.js.map