"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chart_1 = __importDefault(require("./shapes/chart"));
const image_1 = __importDefault(require("./shapes/image"));
const file_1 = __importDefault(require("./helper/file"));
const xml_1 = __importDefault(require("./helper/xml"));
const enums_1 = require("./definitions/enums");
const generic_1 = __importDefault(require("./shapes/generic"));
class Slide {
    constructor(params) {
        this.sourceTemplate = params.template;
        this.sourceNumber = params.number;
        this.sourcePath = `ppt/slides/slide${this.sourceNumber}.xml`;
        this.relsPath = `ppt/slides/_rels/slide${this.sourceNumber}.xml.rels`;
        this.modifications = [];
        this.importElements = [];
    }
    append(targetTemplate) {
        return __awaiter(this, void 0, void 0, function* () {
            this.targetTemplate = targetTemplate;
            this.targetArchive = yield targetTemplate.archive;
            this.targetNumber = targetTemplate.incrementCounter('slides');
            this.targetPath = `ppt/slides/slide${this.targetNumber}.xml`;
            this.targetRelsPath = `ppt/slides/_rels/slide${this.targetNumber}.xml.rels`;
            this.sourceArchive = yield this.sourceTemplate.archive;
            yield this.copySlideFiles();
            yield this.copyRelatedContent();
            yield this.addSlideToPresentation();
            if (this.hasNotes()) {
                yield this.copySlideNoteFiles();
                yield this.updateSlideNoteFile();
                yield this.appendNotesToContentType(this.targetArchive, this.targetNumber);
            }
            if (this.importElements.length) {
                yield this.importedSelectedElements();
            }
            yield this.applyModifications();
        });
    }
    modify(callback) {
        this.modifications.push(callback);
    }
    addSlideToPresentation() {
        return __awaiter(this, void 0, void 0, function* () {
            let relId = yield xml_1.default.getNextRelId(this.targetArchive, 'ppt/_rels/presentation.xml.rels');
            yield this.appendToSlideRel(this.targetArchive, relId, this.targetNumber),
                yield this.appendToSlideList(this.targetArchive, relId),
                yield this.appendSlideToContentType(this.targetArchive, this.targetNumber);
        });
    }
    modifyElement(selector, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let presName = this.sourceTemplate.name;
            let slideNumber = this.sourceNumber;
            this.addElementToModlist(presName, slideNumber, selector, 'modify', callback);
            return this;
        });
    }
    addElement(presName, slideNumber, selector, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            this.addElementToModlist(presName, slideNumber, selector, 'append', callback);
            return this;
        });
    }
    addElementToModlist(presName, slideNumber, selector, mode, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            this.importElements.push({
                presName: presName,
                slideNumber: slideNumber,
                selector: selector,
                mode: mode,
                callback: callback
            });
            return this;
        });
    }
    importedSelectedElements() {
        return __awaiter(this, void 0, void 0, function* () {
            for (let i in this.importElements) {
                let info = yield this.getElementInfo(this.importElements[i]);
                switch (info.type) {
                    case enums_1.ElementType.Chart:
                        yield new chart_1.default(info)[info.mode](this.targetTemplate, this.targetNumber);
                        break;
                    case enums_1.ElementType.Image:
                        yield new image_1.default(info)[info.mode](this.targetTemplate, this.targetNumber);
                        break;
                    case enums_1.ElementType.Shape:
                        yield new generic_1.default(info)[info.mode](this.targetTemplate, this.targetNumber);
                        break;
                }
            }
        });
    }
    getElementInfo(importElement) {
        return __awaiter(this, void 0, void 0, function* () {
            let template = this.root.template(importElement.presName);
            let sourcePath = `ppt/slides/slide${importElement.slideNumber}.xml`;
            let sourceArchive = yield template.archive;
            let sourceElement = yield xml_1.default.findByElementName(sourceArchive, sourcePath, importElement.selector);
            if (!sourceElement) {
                throw new Error(`Can't find ${importElement.selector} on slide ${importElement.slideNumber} in ${importElement.presName}`);
            }
            let appendElementParams = yield this.analyzeElement(sourceElement, sourceArchive, importElement.slideNumber);
            return {
                mode: importElement.mode,
                name: importElement.selector,
                sourceArchive: sourceArchive,
                sourceSlideNumber: importElement.slideNumber,
                sourceElement: sourceElement,
                callback: importElement.callback,
                target: appendElementParams.target,
                type: appendElementParams.type,
            };
        });
    }
    analyzeElement(sourceElement, sourceArchive, slideNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            let isChart = sourceElement.getElementsByTagName('c:chart');
            if (isChart.length) {
                return {
                    type: enums_1.ElementType.Chart,
                    target: yield xml_1.default.getTargetByRelId(sourceArchive, slideNumber, sourceElement, 'chart'),
                };
            }
            let isImage = sourceElement.getElementsByTagName('p:nvPicPr');
            if (isImage.length) {
                return {
                    type: enums_1.ElementType.Image,
                    target: yield xml_1.default.getTargetByRelId(sourceArchive, slideNumber, sourceElement, 'image'),
                };
            }
            return {
                type: enums_1.ElementType.Shape,
            };
        });
    }
    applyModifications() {
        return __awaiter(this, void 0, void 0, function* () {
            for (let m in this.modifications) {
                let xml = yield xml_1.default.getXmlFromArchive(this.targetArchive, this.targetPath);
                this.modifications[m](xml);
                yield xml_1.default.writeXmlToArchive(this.targetArchive, this.targetPath, xml);
            }
        });
    }
    copySlideFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            file_1.default.zipCopy(this.sourceArchive, `ppt/slides/slide${this.sourceNumber}.xml`, this.targetArchive, `ppt/slides/slide${this.targetNumber}.xml`);
            file_1.default.zipCopy(this.sourceArchive, `ppt/slides/_rels/slide${this.sourceNumber}.xml.rels`, this.targetArchive, `ppt/slides/_rels/slide${this.targetNumber}.xml.rels`);
        });
    }
    copySlideNoteFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            file_1.default.zipCopy(this.sourceArchive, `ppt/notesSlides/notesSlide${this.sourceNumber}.xml`, this.targetArchive, `ppt/notesSlides/notesSlide${this.targetNumber}.xml`);
            file_1.default.zipCopy(this.sourceArchive, `ppt/notesSlides/_rels/notesSlide${this.sourceNumber}.xml.rels`, this.targetArchive, `ppt/notesSlides/_rels/notesSlide${this.targetNumber}.xml.rels`);
        });
    }
    updateSlideNoteFile() {
        return __awaiter(this, void 0, void 0, function* () {
            yield xml_1.default.replaceAttribute(this.targetArchive, `ppt/notesSlides/_rels/notesSlide${this.targetNumber}.xml.rels`, "Relationship", "Target", `../slides/slide${this.sourceNumber}.xml`, `../slides/slide${this.targetNumber}.xml`);
            yield xml_1.default.replaceAttribute(this.targetArchive, `ppt/slides/_rels/slide${this.targetNumber}.xml.rels`, "Relationship", "Target", `../notesSlides/notesSlide${this.sourceNumber}.xml`, `../notesSlides/notesSlide${this.targetNumber}.xml`);
        });
    }
    appendToSlideRel(rootArchive, relId, slideCount) {
        return xml_1.default.append({
            archive: rootArchive,
            file: `ppt/_rels/presentation.xml.rels`,
            parent: (xml) => xml.getElementsByTagName('Relationships')[0],
            tag: 'Relationship',
            attributes: {
                Id: relId,
                Type: `http://schemas.openxmlformats.org/officeDocument/2006/relationships/slide`,
                Target: `slides/slide${slideCount}.xml`
            }
        });
    }
    appendToSlideList(rootArchive, relId) {
        return xml_1.default.append({
            archive: rootArchive,
            file: `ppt/presentation.xml`,
            parent: (xml) => xml.getElementsByTagName('p:sldIdLst')[0],
            tag: 'p:sldId',
            attributes: {
                id: (xml) => xml_1.default.getMaxId(xml.getElementsByTagName('p:sldId'), 'id', true),
                'r:id': relId
            }
        });
    }
    appendSlideToContentType(rootArchive, slideCount) {
        return xml_1.default.append(xml_1.default.createContentTypeChild(rootArchive, {
            PartName: `/ppt/slides/slide${slideCount}.xml`,
            ContentType: `application/vnd.openxmlformats-officedocument.presentationml.slide+xml`
        }));
    }
    appendNotesToContentType(rootArchive, slideCount) {
        return xml_1.default.append(xml_1.default.createContentTypeChild(rootArchive, {
            PartName: `/ppt/notesSlides/notesSlide${slideCount}.xml`,
            ContentType: `application/vnd.openxmlformats-officedocument.presentationml.notesSlide+xml`
        }));
    }
    copyRelatedContent() {
        return __awaiter(this, void 0, void 0, function* () {
            let charts = yield chart_1.default.getAllOnSlide(this.sourceArchive, this.relsPath);
            for (let i in charts) {
                yield new chart_1.default({
                    mode: 'append',
                    target: charts[i],
                    sourceArchive: this.sourceArchive,
                    sourceSlideNumber: this.sourceNumber,
                }).modifyOnAddedSlide(this.targetTemplate, this.targetNumber);
            }
            let images = yield image_1.default.getAllOnSlide(this.sourceArchive, this.relsPath);
            for (let i in images) {
                yield new image_1.default({
                    mode: 'append',
                    target: images[i],
                    sourceArchive: this.sourceArchive,
                    sourceSlideNumber: this.sourceNumber,
                }).modifyOnAddedSlide(this.targetTemplate, this.targetNumber);
            }
        });
    }
    hasNotes() {
        let file = this.sourceArchive.file(`ppt/notesSlides/notesSlide${this.sourceNumber}.xml`);
        return file && file.hasOwnProperty('name');
    }
}
exports.default = Slide;
//# sourceMappingURL=slide.js.map